from dataclasses import dataclass
from enum import Enum
from typing import Optional


class BankAccountKind(str, Enum):
    CORRIENTE = "corriente"
    VISTA = "vista"
    AHORRO = "ahorro"
    ELECTRONIC_CHECKBOOK = "electronic_checkbook"
    AR_CBU = "ar_cbu"
    AR_CVU = "ar_cvu"
    AR_ALIAS = "ar_alias "
    BR_CORRIENTE_FISICA = "br_corriente_fisica"
    BR_SIMPLE_FISICA = "br_simple_fisica"
    BR_CORRIENTE_JURIDICA = "br_corriente_juridica"
    BR_POUPANCA_FISICA = "br_poupanca_fisica"
    BR_POUPANCA_JURIDICA = "br_poupanca_juridica"
    BR_CAIXA_FACIL = "br_caixa_facil"
    BR_PIX = "br_pix"


class PixType(str, Enum):
    CPF = "CPF"
    CNPJ = "CNPJ"
    EMAIL = "Email"
    PHONE = "Phone"
    OTRO = "Otro"


@dataclass
class AddClientBankAccountRequest:
    country: str
    bank: str
    number: str
    kind: BankAccountKind
    pix_type: Optional[PixType] = None
    agency: Optional[str] = None
    dv: Optional[str] = None
    province: Optional[str] = None



import simplejson as json
import unittest
from notbank_python_sdk.models.level2_ticker_feed import level2_ticker_feed_from_list


class ConvertLevel2FromJsonTestCase(unittest.TestCase):
    def test_level_2_ticker_from_list(self):
        tickers_as_list_str = "[[1,0,1750691160212,0,104701.54,1,104350,154,0.00047,0],[2,0,1750691160212,0,104701.54,2,104300.00,154,0.00014,0],[3,0,1750691160212,0,104701.54,2,102300,154,0.00013,0],[4,0,1750691160212,0,104701.54,1,105000,154,0.00007,1]]"
        tickers_as_list = json.loads(tickers_as_list_str, use_decimal=True)
        level2_tickers = [level2_ticker_feed_from_list(
            ticker_as_list) for ticker_as_list in tickers_as_list]



import unittest

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.account_fees_request import AccountFeesRequest

from tests import test_helper


class TestGetAccountInfo(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)


    def test_get_account_fees(self):
        account_fees = self.client.get_account_fees(AccountFeesRequest(self.credentials.account_id))
        print(account_fees)


if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_open_orders import GetOpenOrdersRequest
from tests import test_helper


class TestGetOpenOrders(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_open_orders_success(self):
        request = GetOpenOrdersRequest(
            account_id=9,
            instrument_id=1
        )
        response = self.client.get_open_orders(request)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].order_id, 6598)
        self.assertEqual(response[0].quantity, 1.0)
        self.assertEqual(response[0].order_type, "StopMarket")

    def test_get_open_orders_no_results(self):
        request = GetOpenOrdersRequest(
            account_id=10
        )
        response = self.client.get_open_orders(request)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 0)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import List, Optional
from notbank_python_sdk.models.instrument_limits import InstrumentLimits

from notbank_python_sdk.models.product_limits import ProductLimits


@dataclass
class ProductVerificationLevelConfig:
    level: int
    oms_id: int
    products: List[ProductLimits]
    level_name: Optional[str] = None


@dataclass
class InstrumentVerificationLevelConfig:
    level: int
    oms_id: int
    instruments: List[InstrumentLimits]
    level_name: Optional[str] = None



from typing import Callable
from dataclasses import dataclass
from typing import Callable

from notbank_python_sdk.error import NotbankException


@dataclass
class Callback:
    id: str
    builder: Callable[[
        Callable[[NotbankException], None]],
        Callable[[str], None]]



from decimal import Decimal
import unittest
from notbank_python_sdk.error import NotbankException
from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.cancel_replace_order_request import CancelReplaceOrderRequest

from tests import test_helper


class TestSendCancelReplaceList(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_send_cancel_replace_list_success(self):
        """Prueba exitosa, devuelve 'True'."""
        request = [
            CancelReplaceOrderRequest(
                order_id_to_replace=6696,
                order_type="Limit",
                side=0,  # Buy
                account_id=7,
                instrument_id=1,
                quantity=Decimal("0.003"),
                time_in_force=1,  # GTC
            ),
            CancelReplaceOrderRequest(
                order_id_to_replace=6698,
                order_type="Limit",
                side=0,  # Buy
                account_id=7,
                instrument_id=1,
                quantity=Decimal("0.004"),
                time_in_force=1,  # GTC
            ),
        ]

        self.client.send_cancel_replace_list(request)

    def test_send_cancel_replace_list_empty_list(self):
        """Prueba exitosa: lista vacía, devuelve 'False'."""
        request = []

        try:
            self.client.send_cancel_replace_list(request)
        except NotbankException as e:
            self.assertEqual(str(e), "Invalid Request")
            self.assertEqual(e.code, 100)


if __name__ == "__main__":
    unittest.main()



from decimal import Decimal
import unittest
from notbank_python_sdk.error import NotbankException

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_product_request import GetProductRequest
from tests import test_helper


class TestGetProduct(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.client = NotbankClient(connection)

    def test_get_product_success(self):
        """
        Prueba exitosa: solicitud válida, devuelve los detalles del producto.
        """
        request = GetProductRequest(product_id=1)
        response = self.client.get_product(request)

        # Verificaciones
        self.assertIsNotNone(response)
        self.assertEqual(response.oms_id, 1)
        self.assertEqual(response.product_id, 1)
        self.assertEqual(response.product, "USD")
        self.assertEqual(response.product_full_name, "US Dollar")
        self.assertEqual(response.product_type, "NationalCurrency")
        self.assertEqual(response.decimal_places, 2)
        self.assertEqual(response.tick_size, Decimal("0.01"))
        self.assertEqual(response.deposit_enabled, True)
        self.assertEqual(response.withdraw_enabled, True)
        self.assertEqual(response.no_fees, False)
        self.assertEqual(response.is_disabled, False)
        self.assertEqual(response.margin_enabled, False)

    def test_get_product_not_found(self):
        """
        Prueba: product_id inválido, no se encuentra el producto.
        """
        invalid_product_id = 999
        request = GetProductRequest(product_id=invalid_product_id)
        try:
            self.client.get_product(request)
            self.fail()
        except NotbankException:
            pass


if __name__ == "__main__":
    unittest.main()



from time import sleep
import unittest

from notbank_python_sdk.requests_models.subscribe_level1_request import SubscribeLevel1Request
from notbank_python_sdk.requests_models.unsubscribe_level1_request import UnsubscribeLevel1Request
from notbank_python_sdk.notbank_client import NotbankClient
from tests.test_helper import CallMarker, new_websocket_client_connection


class TestSubscribeLevel1Ticker(unittest.TestCase):
    def setUp(self):
        self._websocket_connection = new_websocket_client_connection()
        self._websocket_connection.connect()
        self._system_client = NotbankClient(self._websocket_connection)

    def test_subscribe_with_instrument_id(self):
        snapshot_marker = CallMarker.create()

        self._system_client.subscribe_level_1(
            SubscribeLevel1Request(154),
            lambda ticker: snapshot_marker.mark_called(),
            lambda ticker: None)
        sleep(14)
        self._system_client.unsubscribe_level_1(
            UnsubscribeLevel1Request(154))
        sleep(3)
        self._websocket_connection.close()
        self.assertTrue(snapshot_marker.was_callled(),
                        'snapshot callback was not called')


if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.download_document import DownloadDocumentRequest
from tests import test_helper


class TestDownloadDocument(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_download_document_success(self):
        req = DownloadDocumentRequest("66769552-cacd-471b-bb53-04b1ed1c87f9")
        res = self.client.download_document(req)
        self.assertEqual(res.descriptor_id,
                         "66769552-cacd-471b-bb53-04b1ed1c87f9")
        self.assertEqual(res.status_code, "Success")
        self.assertEqual(res.status_message, "success")
        self.assertTrue(res.doc_name.endswith(".csv"))
        self.assertEqual(res.num_slices, 1)

    def test_download_document_error(self):
        req = DownloadDocumentRequest("invalid-guid")
        res = self.client.download_document(req)
        self.assertEqual(res.status_code, "Error")
        self.assertEqual(res.status_message, "error")
        self.assertEqual(res.doc_name, "")
        self.assertEqual(res.num_slices, 0)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class OmsFee:
    oms_id: int
    account_id: int
    account_provider_id: int
    fee_id: int
    fee_amt: Decimal
    fee_calc_type: str
    is_active: bool
    product_id: int
    minimal_fee_amt: Decimal
    minimal_fee_calc_type: str



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class AccountTrade:
    oms_id: int
    execution_id: int
    trade_id: int
    order_id: int
    account_id: int
    account_name: str
    sub_account_id: int
    client_order_id: int
    instrument_id: int
    side: str
    order_type: str
    quantity: Decimal
    remaining_quantity: Decimal
    price: Decimal
    value: Decimal
    counter_party: str
    order_trade_revision: int
    direction: str
    is_block_trade: bool
    fee: Decimal
    fee_product_id: int
    order_originator: int
    user_name: str
    trade_time_ms: int
    maker_taker: str
    adapter_trade_id: int
    inside_bid: Decimal
    inside_bid_size: Decimal
    inside_ask: Decimal
    inside_ask_size: Decimal
    counter_party_client_user_id: int
    notional_product_id: int
    notional_rate: Decimal
    notional_value: Decimal
    trade_time: int



from typing import Callable, Generic, Optional, TypeVar


L = TypeVar('L')
R = TypeVar('R')
L2 = TypeVar('L2')
R2 = TypeVar('R2')


class Either(Generic[L, R]):
    def __init__(self, left: Optional[L], right: Optional[R]):
        self._left = left
        self._right = right

    @staticmethod
    def right(right: R):
        return Either(None, right)

    @staticmethod
    def left(left: L):
        return Either(left, None)

    def is_right(self) -> bool:
        return self._right is not None

    def is_left(self) -> bool:
        return not self.is_right()

    def get(self) -> R:
        if self._right is None:
            raise ValueError("Cannot get right value from Either.Left")
        return self._right

    def get_left(self) -> L:
        if self._left is None:
            raise ValueError("Cannot get left value from Either.Right")
        return self._left

    def map(self, func: Callable[[R], R2]) -> 'Either[L, R2]':
        if self.is_right():
            return Either.Right(func(self._right))  # type: ignore
        return Either.left(self._left)  # type: ignore

    def map_left(self, func: Callable[[L], L2]) -> 'Either[L2, R]':
        if self.is_left():
            return Either.left(func(self._left))  # type: ignore
        return Either.Right(self._right)  # type: ignore

    def flat_map(self, func: Callable[[R], 'Either[L, R2]']) -> 'Either[L, R2]':
        if self.is_right():
            return func(self._right)  # type: ignore
        return Either.left(self._left)  # type: ignore

    def get_or_else(self, default: R) -> R:
        if self.is_right():
            return self._right  # type: ignore
        return default



from decimal import Decimal
import random
import unittest
from notbank_python_sdk.models.account_positions import AccountPosition
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.models.send_order import  SendOrderStatus
from notbank_python_sdk.requests_models.get_account_positions_request import GetAccountPositionsRequest
from notbank_python_sdk.requests_models.order_book import OrderBookRequest
from notbank_python_sdk.requests_models.send_order import OrderType, SendOrderRequest, Side, TimeInForce
from tests import test_helper


class TestSendOrder(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_send_order_success(self):
        instrument = self.client.get_instrument_by_symbol("BTCUSDT")
        order_price = self._get_order_price()
        order_quantity = self._get_order_quantity()
        request = SendOrderRequest(
            instrument=instrument,
            account_id=self.credentials.account_id,
            time_in_force=TimeInForce.GTD,
            side=Side.Sell,
            quantity=order_quantity,
            limit_price=order_price,
            order_type=OrderType.Limit,
        )
        response = self.client.send_order(request)
        if response.status == SendOrderStatus.REJECTED:
            # order was rejected
            pass
        else:
            # order was accepted
            order_id = response.order_id

    def _get_order_price(self) -> Decimal:
        orderbook = self.client.get_order_book(
            OrderBookRequest("BTCUSDT", 1, 1))
        top_orderbook = orderbook.asks[0]
        delta = Decimal(random.randrange(10, 100))/1000
        order_price = top_orderbook.price + delta
        return order_price

    def _get_order_quantity(self) -> Decimal:
        ust_balance = self._get_balance("USDT")
        total_balance = ust_balance.amount
        return total_balance - \
            Decimal(random.random()) * (-(total_balance/2))

    def _get_balance(self, product_symbol: str) -> AccountPosition:
        positions = self.client.get_account_positions(
            GetAccountPositionsRequest(self.credentials.account_id))
        for position in positions:
            if position.product_symbol == product_symbol:
                return position
        raise Exception("no balance for product: " + product_symbol)

    def test_send_order_not_enough_funds(self):
        instrument = self.client.get_instrument_by_symbol("BTCUSDT")
        request = SendOrderRequest(
            instrument=instrument,
            account_id=self.credentials.account_id,
            time_in_force=TimeInForce.FOK,
            side=Side.Sell,
            quantity=Decimal("12"),
            order_type=OrderType.Market,
        )
        response = self.client.send_order(request)
        self.assertEqual(response.status, "Rejected")
        self.assertEqual(response.errormsg, "Not_Enough_Funds")


if __name__ == "__main__":
    unittest.main()



from queue import Empty, Full, Queue
from typing import Any, Callable, List, Optional, TypeVar

import simplejson as json
from queue import Queue
from typing import Callable, List, TypeVar
from notbank_python_sdk.core.converter import to_json_str
from notbank_python_sdk.core.either import Either
from notbank_python_sdk.error import ErrorCode, NotbankException
from notbank_python_sdk.websocket.callback_manager import CallbackManager

from notbank_python_sdk.websocket.message_frame import MessageFrame
from notbank_python_sdk.websocket.message_type import MessageType
from notbank_python_sdk.websocket.payload_getter import PayloadGetter
from notbank_python_sdk.websocket.subscription import Subscription, Unsubscription
from notbank_python_sdk.websocket.subscription_handler import Callback

T = TypeVar('T')
MessageFrameCallback = Callable[[MessageFrame], None]


class WebsocketRequester:
    _store_subscription_callback: Callable[[str, Callable[[str], None]], None]
    _remove_subscription_callback: Callable[[str], None]
    _store_callback: Callable[[MessageFrameCallback], int]
    _web_socket_send_fn: Callable[[str], None]
    _on_failure: Callable[[Exception], None]
    _request_timeout: Optional[float]

    def __init__(
        self,
        store_subscription_callback: Callable[[str, Callable[[str], None]], None],
        remove_subscription_callback: Callable[[str], None],
        store_callback: Callable[[MessageFrameCallback], int],
        web_socket_send_fn: Callable[[str], None],
        on_failure: Callable[[Exception], None],
        request_timeout: Optional[float] = None
    ):
        self._store_subscription_callback = store_subscription_callback
        self._remove_subscription_callback = remove_subscription_callback
        self._store_callback = store_callback
        self._web_socket_send_fn = web_socket_send_fn
        self._on_failure = on_failure
        self._request_timeout = request_timeout

    @staticmethod
    def create(
            callback_manager: CallbackManager,
            web_socket_send_fn: Callable[[str], None],
            on_failure: Callable[[Exception], None],
            request_timeout: Optional[float] = None,
    ) -> 'WebsocketRequester':
        return WebsocketRequester(
            store_subscription_callback=callback_manager.subscription_callbacks.add,
            remove_subscription_callback=callback_manager.subscription_callbacks.remove,
            store_callback=callback_manager.sequenced_callbacks.put,
            web_socket_send_fn=web_socket_send_fn,
            on_failure=on_failure,
            request_timeout=request_timeout,
        )

    def subscribe(self, subscription: Subscription[Any]) -> Callable[[], Either[NotbankException, str]]:
        for callback in subscription.callbacks:
            self._store_subscription_callback(
                callback.id, callback.builder(self._on_failure))
        return self._request_to_queue(subscription.endpoint, subscription.message, MessageType.REQUEST)

    def unsubscribe(self, unsubscription: Unsubscription):
        for callback_id in unsubscription.callback_ids:
            self._remove_subscription_callback(callback_id)
        return self._request_to_queue(unsubscription.endpoint, unsubscription.message, MessageType.REQUEST)

    def request(self, endpoint: str, message: str) -> Callable[[], Either[NotbankException, str]]:
        return self._request_to_queue(endpoint, message, MessageType.REQUEST)

    def _request_to_queue(self, endpoint: str, message: str, message_type: MessageType) -> Callable[[], Either[NotbankException, str]]:
        response_buffer = Queue(maxsize=1)
        sequence_number = self._store_callback(
            self._handle_request_response(response_buffer))
        message_frame = MessageFrame(
            message_type, sequence_number, endpoint, json.dumps(message, use_decimal=True))
        message_frame_str = to_json_str(
            message_frame, {MessageType: lambda message_type: message_type.value})
        self._web_socket_send_fn(message_frame_str)

        def wait_response() -> Either[NotbankException, str]:
            try:
                return response_buffer.get(timeout=self._request_timeout)
            except Empty:
                return Either.left(NotbankException(ErrorCode.TIMED_OUT, "request timed out"))
        return wait_response

    def _handle_request_response(self, response_buffer: Queue):
        def response_handler(message_frame: MessageFrame):
            data = PayloadGetter.get(message_frame)
            try:
                response_buffer.put(data, block=False)
            except Full:
                pass
        return response_handler



from typing import Callable, Optional, TypeVar
from notbank_python_sdk.models.authenticate_response import AuthenticateResponse
from notbank_python_sdk.websocket.websocket_connection import WebsocketConnection

T = TypeVar('T')


class Reauther:
    _reauthenticate_connection: Callable[[
        WebsocketConnection], Optional[AuthenticateResponse]]

    def __init__(self):
        self._reauthenticate_connection = lambda x: None

    def update_authenticate_fn(self, authenticate_connection: Callable[[WebsocketConnection], Optional[AuthenticateResponse]]):
        self._reauthenticate_connection = authenticate_connection

    def reauthenticate(self, connection: WebsocketConnection) -> Optional[AuthenticateResponse]:
        return self._reauthenticate_connection(connection)



import json
from dataclasses import dataclass
import logging
from typing import Any, Callable, Optional
from notbank_python_sdk.client_connection import ClientConnection
from notbank_python_sdk.client_connection_factory import (
    new_websocket_client_connection as create_new_websocket_client_connection, new_rest_client_connection as create_new_rest_client_connection)
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest


TEST_URL = "stgapi.notbank.exchange"


@dataclass
class Credentials:
    public_key: str
    secret_key: str
    user_id: int
    account_id: int


def load_credentials(file_path: Optional[str] = None) -> Credentials:
    if file_path is None:
        file_path = 'keys.json'
    with open(file_path) as f:
        data = json.load(f)
        return Credentials(
            public_key=data['ApiPublicKey'],
            secret_key=data['ApiSecretKey'],
            user_id=data['UserId'],
            account_id=data['AccountId'],
        )


def authenticate_connection(connection: ClientConnection, credentials: Optional[Credentials] = None) -> bool:
    if credentials is None:
        credentials = load_credentials('keys.json')
    response = connection.authenticate(AuthenticateRequest(
        api_public_key=credentials.public_key,
        api_secret_key=credentials.secret_key,
        user_id=credentials.user_id,
    ))
    return response.authenticated


def new_websocket_client_connection():
    logging.basicConfig()
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    return create_new_websocket_client_connection(
        TEST_URL,
        lambda: logger.debug("on open"),
        lambda o1, o2: logger.debug("on close"),
        lambda err: logger.debug("error: " + str(err)),
        lambda msg: logger.debug("message in: " + msg),
        lambda msg: logger.debug("message out: "+msg),
        5)


def print_message_out(httpMethod: str, headers: Any, extra_headers: Any, url: str, body: dict) -> None:
    print("\n", "** message out+*")
    print("httpMethod:", httpMethod)
    # print("headers:", headers)
    # print("extra headers:", extra_headers)
    print("url:", url)
    print("body:", body)


def print_message_in(body: str) -> None:
    print("\n", "** message in**")
    print("body:", body)


def new_rest_client_connection(
    peek_message_in: Callable[[str], None] = lambda a: None,
    peek_message_out: Callable[[str, Any, Any, str,
                                dict], None] = lambda a, b, c, d, e: None
):
    return create_new_rest_client_connection(TEST_URL, peek_message_in, peek_message_out)


class CallMarker:
    _call_count: int

    def __init__(self, call_count: int) -> None:
        self._call_count = call_count

    @staticmethod
    def create() -> 'CallMarker':
        return CallMarker(0)

    def mark_called(self) -> None:
        self._call_count += 1

    def was_callled(self) -> bool:
        return self._call_count > 0

    def get_call_count(self) -> int:
        return self._call_count



from enum import Enum
from typing import Any, Callable, TypeVar

from notbank_python_sdk.error import ErrorCode, NotbankException, StandardErrorResponse
from dacite.data import Data
from dacite import Config, MissingValueError, from_dict as dacite_from_dict

T = TypeVar('T')


class ApDataHandler:
    @staticmethod
    def handle_ap_data(parse_response: Callable[[Data], T], response_data: Any) -> T:
        try:
            standard_response = dacite_from_dict(
                StandardErrorResponse,
                response_data,
                config=Config(cast=[Enum]))
            if standard_response.result is False:
                raise NotbankException.create(standard_response)
        except MissingValueError:
            pass
        try:
            return parse_response(response_data)
        except MissingValueError as e:
            raise NotbankException(
                ErrorCode.CONFIGURATION_ERROR,
                f"notbank sdk badly configured. {e}")



from enum import Enum
from typing import Any, Callable, Dict, List, TypeVar

from notbank_python_sdk.error import ErrorCode, NotbankException
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest
from notbank_python_sdk.models.authenticate_response import AuthenticateResponse
from notbank_python_sdk.websocket.subscription import Subscription, Unsubscription
from notbank_python_sdk.core.endpoint_category import EndpointCategory


T = TypeVar('T')

ParseResponseFn = Callable[[Any], T]
ParseResponseListFn = Callable[[Any], List[T]]
RequestMethodFn = Callable[[str, EndpointCategory, Any, ParseResponseFn[T]], T]
AuthenticateMethodFn = Callable[[AuthenticateRequest], AuthenticateResponse]
SubscribeFn = Callable[[Subscription[T]], T]
UnsubscribeFn = Callable[[Unsubscription[T]], T]


class RequestType(Enum):
    POST = 0
    GET = 1
    DELETE = 2


class ClientConnection:
    def __init__(
        self,
        post_request: RequestMethodFn,
        get_request: RequestMethodFn,
        delete_request: RequestMethodFn,
        subscribe: SubscribeFn,
        unsubscribe: UnsubscribeFn,
        authenticate_user: AuthenticateMethodFn,
        connect: Callable[[], None],
        close: Callable[[], None]
    ):
        self._request_methods: Dict[RequestType, RequestMethodFn] = {
            RequestType.POST: post_request,
            RequestType.GET: get_request,
            RequestType.DELETE: delete_request,
        }
        self._subscribe = subscribe
        self._unsubscribe = unsubscribe
        self._authenticate_user = authenticate_user
        self._connect = connect
        self._close = close

    def request(
        self,
        endpoint: str,
        endpoint_category: EndpointCategory,
        request_data: Any,
        parse_response_fn: ParseResponseFn[T],
        request_type: RequestType = RequestType.POST,
    ) -> T:
        request_method = self._request_methods.get(request_type)
        if request_method is None:
            raise NotbankException(
                ErrorCode.CONFIGURATION_ERROR, f"no request method found for type: {request_type}")
        return request_method(endpoint, endpoint_category, request_data, parse_response_fn)

    def subscribe(self, subscription: Subscription[T]) -> T:
        return self._subscribe(subscription)

    def unsubscribe(self, unsubscription: Unsubscription[T]) -> T:
        return self._unsubscribe(unsubscription)

    def connect(self) -> None:
        self._connect()

    def close(self) -> None:
        self._close()

    def authenticate(self, authenticate_request: AuthenticateRequest) -> AuthenticateResponse:
        return self._authenticate_user(authenticate_request)



from typing import Any, Callable, Dict, List, Type, TypeVar

from notbank_python_sdk.client_connection import ParseResponseFn, ParseResponseListFn
from notbank_python_sdk.core.either import Either
from notbank_python_sdk.core.converter import from_dict
from notbank_python_sdk.error import ErrorCode, NotbankException

T1 = TypeVar('T1')
T2 = TypeVar('T2')


def build_subscription_handler(
    handler: Callable[[T1], None],
    payload_parser: Callable[[str], Either[NotbankException, T1]]
) -> Callable[[Callable[[NotbankException], None]], Callable[[str], None]]:
    def subscription_handler_builder(
        on_failure: Callable[[NotbankException], None]
    ) -> Callable[[str], None]:
        def subscription_handler(payload_str: str) -> None:
            payload = payload_parser(payload_str)
            if payload.is_left():
                on_failure(payload.get_left())
                return
            try:
                handler(payload.get())
            except Exception as e:
                on_failure(NotbankException(ErrorCode.UNKNOWN, str(e)))
        return subscription_handler
    return subscription_handler_builder


def parse_response_fn(response_cls: Type[T2], no_pascal_case: List[str] = [], overrides: Dict[str, str] = {}, from_pascal_case: bool = True) -> ParseResponseFn[T2]:
    def parse_data_fn(data: Any) -> T2:
        return from_dict(response_cls, data, no_pascal_case, overrides, from_pascal_case)
    return parse_data_fn


def parse_response_list_fn(response_cls: Type[T2], no_pascal_case: List[str] = [], overrides: Dict[str, str] = {}, from_pascal_case: bool = True) -> ParseResponseListFn[T2]:
    def parse_data_fn(data: Any) -> List[T2]:
        return [from_dict(response_cls, elem, no_pascal_case, overrides, from_pascal_case) for elem in data]
    return parse_data_fn



from dataclasses import dataclass
from decimal import Decimal
from typing import List


@dataclass
class Level2TickerSnapshot:
    md_update_id: int
    number_of_unique_accounts: int
    action_date_time: int
    action_type: int
    last_trade_price: Decimal
    number_of_orders: int
    price: Decimal
    product_pair_code: int
    quantity: Decimal
    side: int



from decimal import Decimal
import unittest
from uuid import UUID
from notbank_python_sdk.requests_models.create_fiat_deposit_request import CreateFiatDepositRequest
from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.get_banks_request import GetBanksRequest


class TestCreateFiatDeposit(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_create_fiat_deposit(self):
        self.client.create_fiat_deposit(CreateFiatDepositRequest(
            account_id=self.credentials.account_id,
            payment_method=1,
            currency="ARS",
            amount=Decimal("10"),
            bank_account_id=UUID("4d677d9c-81e1-45d2-9903-43fd599b6599")
        ))


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Optional, Union

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetProductsRequest(WithOMSId):
    attribute: Optional[str] = None
    attribute_value: Optional[str] = None
    get_disabled: Optional[Union[bool, int]] = None
    depth: Optional[int] = None
    start_index: Optional[int] = None



from dataclasses import dataclass
from typing import Union
from uuid import UUID


@dataclass
class ResendVerificationCodeWhitelistedAddress:
    whitelisted_address_id: UUID
    account_id: int


@dataclass
class ResendVerificationCodeWhitelistedAddressInternal:
    account_id: int



import unittest
from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.generate_trade_activity_report import GenerateTradeActivityReportRequest
from tests import test_helper


class TestGenerateTradeActivityReport(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_generate_trade_activity_report_success(self):
        request = GenerateTradeActivityReportRequest(
            start_time="2023-03-01T16:00:00.000Z",
            end_time="2023-03-02T16:00:00.000Z",
            account_id_list=[185, 9],
        )
        response = self.client.generate_trade_activity_report(request)
        self.assertEqual(response.requesting_user, 6)
        self.assertEqual(response.oms_id, 1)
        self.assertEqual(response.report_flavor, "TradeActivity")
        self.assertEqual(response.request_status, "Submitted")
        self.assertEqual(response.account_ids, [9, 185])

    def test_generate_trade_activity_report_empty_accounts(self):
        request = GenerateTradeActivityReportRequest(
            start_time="2024-01-01T00:00:00.000Z",
            end_time="2024-01-02T00:00:00.000Z",
            account_id_list=[],
        )
        response = self.client.generate_trade_activity_report(request)
        self.assertEqual(response.account_ids, [])
        self.assertEqual(response.request_status, "Submitted")
        self.assertEqual(response.report_flavor, "TradeActivity")

    def test_generate_trade_activity_report_invalid_oms(self):
        request = GenerateTradeActivityReportRequest(
            start_time="2024-01-01T00:00:00.000Z",
            end_time="2024-01-02T00:00:00.000Z",
            account_id_list=[1],
        )
        with self.assertRaises(Exception) as context:
            self.client.generate_trade_activity_report(request)
        self.assertIn("Invalid OMSId", str(context.exception))


if __name__ == "__main__":
    unittest.main()




import simplejson as json
import unittest
from notbank_python_sdk.core import converter
from notbank_python_sdk.models.account_info import AccountInfo


class ConvertTickerFromJsonTestCase(unittest.TestCase):
    def test_ticker_from_list(self):
        account_info_json_str = "{\"OMSID\":1,\"AccountId\":235,\"AccountName\":\"Primary | ismael@dysopsis.com\",\"AccountHandle\":null,\"FirmId\":null,\"FirmName\":null,\"AccountType\":\"Asset\",\"FeeGroupId\":1,\"ParentID\":0,\"RiskType\":\"Normal\",\"MarginAccountStatus\":\"Active\",\"VerificationLevel\":0,\"VerificationLevelName\":null,\"CreditTier\":0,\"FeeProductType\":\"BaseProduct\",\"FeeProduct\":0,\"RefererId\":0,\"LoyaltyProductId\":0,\"LoyaltyEnabled\":false,\"PriceTier\":0,\"Frozen\":false,\"DateTimeUpdated\":\"2025-07-08T22:10:57.357Z\",\"DateTimeCreated\":\"2025-07-08T22:10:57.357Z\"}"
        account_info = converter.from_json_str(
            AccountInfo, account_info_json_str, overrides={"oms_id": "OMSID"})
        self.assertTrue(account_info.is_right())



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_products_request import GetProductsRequest
from tests import test_helper


class TestGetProducts(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_products_success(self):
        """
        Prueba exitosa: obtiene una lista de productos válidos.
        """
        products = self.client.get_products()

        # Verificamos que la respuesta contiene dos productos
        self.assertEqual(len(products), 2)

        # Verificamos los detalles del primer producto
        self.assertEqual(products[0].product, "USD")
        self.assertEqual(products[0].product_type, "NationalCurrency")
        self.assertEqual(products[0].decimal_places, 2)
        self.assertEqual(products[0].tick_size, 0.01)
        self.assertEqual(products[0].deposit_enabled, True)
        self.assertEqual(products[0].withdraw_enabled, True)
        self.assertEqual(products[0].no_fees, False)
        self.assertEqual(products[0].is_disabled, False)
        self.assertEqual(products[0].margin_enabled, False)

        # Verificamos los detalles del segundo producto
        self.assertEqual(products[1].product, "BTC")
        self.assertEqual(products[1].product_type, "CryptoCurrency")
        self.assertEqual(products[1].decimal_places, 8)
        self.assertEqual(products[1].tick_size, 0.00000001)
        self.assertEqual(products[1].deposit_enabled, True)
        self.assertEqual(products[1].withdraw_enabled, True)
        self.assertEqual(products[1].no_fees, False)
        self.assertEqual(products[1].is_disabled, False)
        self.assertEqual(products[1].margin_enabled, False)

if __name__ == "__main__":
    unittest.main()



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_orders import GetOrdersRequest
from tests import test_helper


class TestGetOrders(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_orders_success(self):
        request = GetOrdersRequest(
            account_id=7,
        )
        response = self.client.get_orders(request)

        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].order_id, 6713)
        self.assertEqual(response[0].price, 0.0)
        self.assertEqual(response[0].quantity, 0.0)
        self.assertEqual(response[0].order_state, "FullyExecuted")
        self.assertEqual(response[0].avg_price, 6000.0)

    def test_get_orders_not_found(self):
        request = GetOrdersRequest(
            account_id=999,
        )
        response = self.client.get_orders(request)
        self.assertEqual(len(response), 0)

    def test_invalid_oms_id(self):
        request = GetOrdersRequest(
            account_id=7,
        )

        with self.assertRaises(Exception) as context:
            self.client.get_orders(request)

        self.assertIn("Endpoint desconocido", str(context.exception))


if __name__ == "__main__":
    unittest.main()



from typing import Any, Callable


class WebsocketHandler:
    def __init__(
        self,
        handle: Callable[[Any], None],
        on_open: Callable[[], None] = lambda: None,
        on_close: Callable[[Any, str], None] = lambda code, message: None,
        on_failure: Callable[[Exception], None] = lambda e: None,
    ) -> None:
        self._handle = handle
        self._on_open = on_open
        self._on_close = on_close
        self._on_failure = on_failure

    def handle(self, data: Any) -> None:
        self._handle(data)

    def on_open(self) -> None:
        self._on_open()

    def on_close(self, code: int, message: str) -> None:
        self._on_close(code, message)

    def on_error(self, error: Exception) -> None:
        self._on_failure(error)



from typing import Any, Callable, List, Optional, TypeVar

import requests
from dacite.data import Data

from notbank_python_sdk.error import ErrorCode, NotbankException
from notbank_python_sdk.models.authenticate_response import AuthenticateResponse
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest
from notbank_python_sdk.core.authenticator import Authenticator
from notbank_python_sdk.core.converter import from_dict, to_dict
from notbank_python_sdk.core.response_handler import ResponseHandler
from notbank_python_sdk.core.endpoint_category import EndpointCategory

T = TypeVar('T')
ParseResponseFn = Callable[[Any], T]

AUTHENTICATE_USER_ENDPOINT = "AuthenticateUser"
AUTHENTICATE_2FA = "Authenticate2FA"


class RestClientConnection:
    NAME = "Notbank"
    VERSION = "0.0.1"
    _host: str
    _rest_session: requests.Session
    _peek_message_in: Callable[[str], None]
    _peek_message_out: Callable[[str, Any, Any, str, dict], None]

    def __init__(
            self,
            host: str,
            ap_token: Optional[str] = None,
            peek_message_in: Callable[[str], None] = lambda a: None,
            peek_message_out: Callable[[str, Any, Any, str, dict], None] = lambda a, b, c, d, e: None,
    ):
        self._host = self._get_host_url(host)
        self._rest_session = requests.Session()
        self._rest_session.headers.update({
            'Content-Type': 'application/json',
            'User-Agent': f'{self.NAME} Python SDK v{self.VERSION}',
        })
        self._update_headers(ap_token)
        self._peek_message_in = peek_message_in
        self._peek_message_out = peek_message_out

    def _get_host_url(self, host: str) -> str:
        return "https://" + host

    def _update_headers(self, ap_token: Optional[str], header_to_remove_list: List[str] = []) -> None:
        for header_to_remove in header_to_remove_list:
            del self._rest_session.headers[header_to_remove]
        if ap_token is not None:
            self._rest_session.headers.update({
                'aptoken': ap_token,
            })

    def close(self) -> None:
        self._rest_session.close()

    def _get_endpoint_url(self, endpoint: str, endpoint_category: EndpointCategory,) -> str:
        url = self._host + "/" + endpoint_category.val + "/" + endpoint
        return url

    def get(self, endpoint: str, endpoint_category: EndpointCategory, params: Any, parse_response: ParseResponseFn[T]) -> T:
        url = self._get_endpoint_url(endpoint, endpoint_category)
        self._peek_message_out(
            "get", self._rest_session.headers, {}, url, params)
        response = self._rest_session.get(url, params=params)
        return self.handle_response(endpoint_category, response, parse_response)

    def post(self, endpoint: str, endpoint_category: EndpointCategory, json_data: Any, parse_response: ParseResponseFn[T], headers: dict = {}) -> T:
        url = self._get_endpoint_url(endpoint, endpoint_category)
        self._peek_message_out(
            "post", self._rest_session.headers, headers, url, json_data)
        response = self._rest_session.post(
            url, json=json_data, headers=headers)
        return self.handle_response(endpoint_category, response, parse_response)

    def delete(self, endpoint: str, endpoint_category: EndpointCategory, params: Any, parse_response: ParseResponseFn[T]) -> T:
        url = self._get_endpoint_url(endpoint, endpoint_category)
        self._peek_message_out(
            "delete", self._rest_session.headers, {}, url, params)
        response = self._rest_session.delete(url, json=params)
        return self.handle_response(endpoint_category, response, parse_response)

    def handle_response(self, endpoint_category: EndpointCategory,response : requests.Response, parse_response: ParseResponseFn[T]) -> T:
        self._peek_message_in(response.text)
        return ResponseHandler.handle_response(
            endpoint_category,
            parse_response,
            response)

    def authenticate_user(self, authenticate_request: AuthenticateRequest) -> AuthenticateResponse:
        request_data = Authenticator.convert_data(authenticate_request)
        headers = to_dict(request_data)
        self._rest_session.headers.update(headers)
        auth_response = self.get(
            AUTHENTICATE_USER_ENDPOINT,
            EndpointCategory.AP,
            {},
            lambda response_data: from_dict(AuthenticateResponse, response_data))
        if not auth_response.authenticated:
            raise NotbankException(
                ErrorCode.OPERATION_FAILED,
                auth_response.errormsg if auth_response.errormsg else "unable to authenticate")
        self._update_headers(ap_token=auth_response.session_token,
                             header_to_remove_list=list(headers.keys()))
        return auth_response



from typing import Callable, TypeVar, Generic, List

from notbank_python_sdk.websocket.message_frame import MessageFrame
from .sequenced_callbacks import SequencedCallbacks
from .subscription_callbacks import SubscriptionCallbacks

T = TypeVar('T')


class CallbackManager:
    sequenced_callbacks: SequencedCallbacks[MessageFrame]
    subscription_callbacks: SubscriptionCallbacks

    def __init__(self, sequenced_callbacks: SequencedCallbacks[MessageFrame], subscription_callbacks: SubscriptionCallbacks):
        self.sequenced_callbacks = sequenced_callbacks
        self.subscription_callbacks = subscription_callbacks

    @staticmethod
    def create() -> 'CallbackManager':
        return CallbackManager(SequencedCallbacks.create(), SubscriptionCallbacks.create())

    def close(self) -> None:
        self.sequenced_callbacks.close()
        self.subscription_callbacks.close()



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_level1_summary import GetLevel1SummaryRequest
from tests import test_helper


class TestGetLevel1Summary(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_valid_oms_id(self):
        """
        Prueba: Obtener el resumen de nivel 1 para un OMSId válido.
        """
        response = self.client.get_level1_summary()

        # Validaciones
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].instrument_id, 1)
        self.assertEqual(response[0].best_bid, 30000)
        self.assertEqual(response[0].last_traded_px, 29354.89)

    def test_empty_response(self):
        """
        Prueba: Obtener resumen de nivel 1 para un OMSId sin registros.
        """
        response = self.client.get_level1_summary()

        # Validaciones
        self.assertEqual(len(response), 0)

    def test_invalid_oms_id(self):
        """
        Prueba: Obtener resumen de nivel 1 para un OMSId inválido.
        """
        with self.assertRaises(Exception) as context:
            self.client.get_level1_summary()

        # Validación del mensaje de error
        self.assertEqual(str(context.exception), "Invalid OMSId")


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal
from enum import IntEnum
from typing import Optional

from notbank_python_sdk.core.tools import truncate_dec
from notbank_python_sdk.models.instrument import Instrument
from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


class TimeInForce(IntEnum):
    GTC = 1
    OPG = 2
    IOC = 3
    FOK = 4
    GTX = 5
    GTD = 6


class Side(IntEnum):
    Buy = 0
    Sell = 1


class OrderType(IntEnum):
    Market = 1
    Limit = 2
    StopMarket = 3
    StopLimit = 4
    TrailingStopMarket = 5
    TrailingStopLimit = 6
    BlockTrade = 7


class PegPriceType(IntEnum):
    Last = 1
    Bid = 2
    Ask = 3


@dataclass
class SendOrderRequest(WithOMSId):
    instrument: Instrument
    account_id: int
    time_in_force: TimeInForce
    side: Side
    order_type: OrderType
    quantity: Decimal
    client_order_id: Optional[int] = None
    order_id_oco: Optional[int] = None
    use_display_quantity: Optional[bool] = None
    peg_price_type: Optional[PegPriceType] = None
    limit_price: Optional[Decimal] = None
    stop_price: Optional[Decimal] = None
    trailing_amount: Optional[Decimal] = None
    limit_offset: Optional[Decimal] = None
    display_quantity: Optional[int] = None
    value: Optional[Decimal] = None


@dataclass
class SendOrderRequestInternal:
    oms_id: int
    instrument_id: int
    account_id: int
    time_in_force: int
    side: int
    order_type: int
    quantity: Decimal
    client_order_id: Optional[int] = None
    order_id_oco: Optional[int] = None
    use_display_quantity: Optional[bool] = None
    peg_price_type: Optional[int] = None
    limit_price: Optional[Decimal] = None
    stop_price: Optional[Decimal] = None
    trailing_amount: Optional[Decimal] = None
    limit_offset: Optional[Decimal] = None
    display_quantity: Optional[int] = None
    value: Optional[Decimal] = None

    @staticmethod
    def from_send_order_request(order_request: SendOrderRequest) -> 'SendOrderRequestInternal':
        instrument = order_request.instrument

        quantity = truncate_dec(order_request.quantity,
                                instrument.quantity_increment)

        stop_price = order_request.stop_price
        if stop_price is not None:
            stop_price = truncate_dec(stop_price, instrument.price_increment)

        limit_price = order_request.limit_price
        if limit_price is not None:
            limit_price = truncate_dec(limit_price, instrument.price_increment)

        return SendOrderRequestInternal(
            oms_id=order_request.oms_id,
            instrument_id=instrument.instrument_id,
            account_id=order_request.account_id,
            time_in_force=int(order_request.time_in_force),
            side=int(order_request.side),
            order_type=int(order_request.order_type),
            client_order_id=order_request.client_order_id,
            order_id_oco=order_request.order_id_oco,
            use_display_quantity=order_request.use_display_quantity,
            quantity=quantity,
            peg_price_type=int(
                order_request.peg_price_type) if order_request.peg_price_type is not None else None,
            limit_price=limit_price,
            stop_price=stop_price,
            trailing_amount=order_request.trailing_amount,
            limit_offset=order_request.limit_offset,
            display_quantity=order_request.display_quantity,
            value=order_request.value,
        )



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional


@dataclass
class ProductLimits:
    oms_id: int
    verification_level: int
    product_id: int
    product_name: str
    auto_withdraw_threshold: Decimal
    daily_deposit_limit: Decimal
    daily_deposit_notional_limit: Decimal
    monthly_deposit_limit: Decimal
    monthly_deposit_notional_limit: Decimal
    yearly_deposit_limit: Decimal
    yearly_deposit_notional_limit: Decimal
    daily_withdraw_limit: Decimal
    daily_withdraw_notional_limit: Decimal
    monthly_withdraw_limit: Decimal
    monthly_withdraw_notional_limit: Decimal
    yearly_withdraw_limit: Decimal
    yearly_withdraw_notional_limit: Decimal
    daily_transfer_notional_limit: Decimal
    notional_product_id: int
    over_limit_rejected: bool
    withdraw_processing_delay_sec: int
    deposit_ticket_workflow: str
    withdraw_ticket_workflow: str
    require_whitelisted_address: bool
    auto_accept_whitelisted_address: bool
    verification_level_name: Optional[str] = None



from typing import Any, Callable, Generic, List, TypeVar

from notbank_python_sdk.websocket.subscription_handler import Callback

T = TypeVar('T')


class Subscription(Generic[T]):
    endpoint: str
    message: Any
    callbacks: List[Callback]
    parse_response_fn: Callable[[Any], T]

    def __init__(
        self,
        endpoint: str,
        message: Any,
        callbacks: List[Callback],
        parse_response_fn: Callable[[Any], T],
    ):
        self.endpoint = endpoint
        self.message = message
        self.callbacks = callbacks
        self.parse_response_fn = parse_response_fn


class Unsubscription(Generic[T]):
    endpoint: str
    message: Any
    callback_ids: List[str]
    parse_response_fn: Callable[[Any], T]

    def __init__(
        self,
        endpoint: str,
        message: Any,
        callback_ids: List[str],
        parse_response_fn: Callable[[Any], T],
    ):
        self.endpoint = endpoint
        self.message = message
        self.callback_ids = callback_ids
        self.parse_response_fn = parse_response_fn



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.user_accounts import GetUserAccountsRequest
from tests import test_helper


class TestGetUserAccounts(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_user_accounts_single(self):
        request = GetUserAccountsRequest(user_id=1)
        response = self.client.get_user_accounts(request)
        self.assertIsInstance(response, list)

    def test_get_user_accounts_multiple(self):
        request = GetUserAccountsRequest(user_id=5)
        response = self.client.get_user_accounts(request)
        self.assertEqual(response, [7, 8, 9])

    def test_get_user_accounts_default_user(self):
        request = GetUserAccountsRequest()
        response = self.client.get_user_accounts(request)
        self.assertEqual(response, [6])

    def test_get_user_accounts_no_result(self):
        request = GetUserAccountsRequest(user_id=999)
        response = self.client.get_user_accounts(request)
        self.assertEqual(response, [])


if __name__ == "__main__":
    unittest.main()



from time import sleep
import unittest

from notbank_python_sdk.requests_models.subscribe_trades_request import SubscribeTradesRequest
from notbank_python_sdk.requests_models.unsubscribe_trades_request import UnsubscribeTradesRequest
from notbank_python_sdk.notbank_client import NotbankClient
from tests.test_helper import CallMarker, new_websocket_client_connection


class TestSubscribeTrades(unittest.TestCase):
    def setUp(self):
        self._websocket_connection = new_websocket_client_connection()
        self._websocket_connection.connect()
        self.client = NotbankClient(self._websocket_connection)

    def test_subscribe_with_instrument_id(self):
        snapshot_marker = CallMarker.create()
        update_marker = CallMarker.create()

        self.client.subscribe_trades(
            SubscribeTradesRequest(154, 3),
            lambda trades: snapshot_marker.mark_called(),
            lambda trades: update_marker.mark_called())
        sleep(60)
        self.client.unsubscribe_trades(
            UnsubscribeTradesRequest(154))
        self._websocket_connection.close()
        self.assertTrue(snapshot_marker.was_callled(),
                        'snapshot callback was not called')
        self.assertTrue(update_marker.was_callled(),
                        'update callback was not called')


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from enum import IntEnum
from typing import Optional, List

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


class TransactionTypes(IntEnum):
    FEE = 1
    TRADE = 2
    OTHER = 3
    REVERSE = 4
    HOLD = 5
    REBATE = 6
    MARGIN_ACQUISITION = 7
    MARGIN_RELINQUISH_BY_TRADE = 8
    MARGIN_INTEREST_TRANSFER = 9
    MARGIN_OPERATOR_TRANSFER_TO_MARGIN_ACCOUNT = 10
    MARGIN_OPERATOR_TRANSFER_TO_ASSET_ACCOUNT = 11
    MARGIN_USER_TRANSFER = 12
    MARGIN_RELINQUISH_BY_TRANSFER = 13
    MARGIN_RELINQUISH_BY_REVERSE_TRADE = 14
    DISTRIBUTION = 15
    PAYMENT = 16
    OPERATOR_LEND = 21
    OPERATOR_RECEIVED = 22
    REBALANCE = 23
    COMMISSION = 24
    AIR_DROP = 25


class TransactionReferenceTypes(IntEnum):
    TRADE = 1
    DEPOSIT = 2
    WITHDRAW = 3
    TRANSFER = 4
    ORDER_HOLD = 5
    WITHDRAW_HOLD = 6
    DEPOSIT_HOLD = 7
    MARGIN_HOLD = 8
    MANUAL_HOLD = 9
    MANUAL_ENTRY = 10
    MARGIN_ACQUISITION = 11
    MARGIN_RELINQUISH = 12
    MARGIN_INTEREST_NETTING = 13
    MARGIN_OPERATOR_TRANSFER_TO_MARGIN_ACCOUNT = 14
    MARGIN_OPERATOR_TRANSFER_TO_ASSET_ACCOUNT = 15
    MARGIN_USER_TRANSFER = 16
    MARGIN_POSITION_REVERSE_TRADE = 17
    AFFILIATE_REBATE = 18
    DISTRIBUTION_ENTRY = 19
    TRANSFER_HOLD = 20
    AIR_DROP = 21


@dataclass
class GetAccountTransactionsRequest(WithOMSId):
    account_id: Optional[int] = None
    depth: Optional[int] = None
    product_id: Optional[int] = None
    transaction_id: Optional[int] = None
    reference_id: Optional[int] = None
    transaction_types: Optional[List[TransactionTypes]] = None
    transaction_reference_types: Optional[List[TransactionReferenceTypes]] = None
    start_timestamp: Optional[str] = None
    end_timestamp: Optional[str] = None



from notbank_python_sdk.core.converter import from_json_str
from notbank_python_sdk.core.either import Either
from notbank_python_sdk.error import ErrorCode,  NotbankException, StandardErrorResponse

from notbank_python_sdk.websocket.message_frame import MessageFrame
from notbank_python_sdk.websocket.message_type import MessageType


class PayloadGetter:
    @staticmethod
    def get(message_frame: MessageFrame) -> Either[NotbankException, str]:
        standard_error_response = from_json_str(
            StandardErrorResponse, message_frame.o)
        if standard_error_response.is_right() and standard_error_response.get().result == False:
            return Either.left(NotbankException.create(standard_error_response.get()))
        is_error_type = message_frame.m == MessageType.ERROR
        if is_error_type:
            return Either.left(NotbankException(ErrorCode.UNKNOWN, message_frame.o))
        return Either.right(message_frame.o)




from dataclasses import dataclass
from decimal import Decimal
from typing import Optional


@dataclass
class TransferFundsRequest:
    sender_account_id: int
    receiver_account_id: int
    currency_name: str
    amount: Decimal
    otp: Optional[str] = None
    notes: Optional[str] = None



from dataclasses import dataclass
from typing import List

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GenerateTransactionActivityReportRequest(WithOMSId):
    start_time: str
    end_time: str
    account_id_list: List[int]



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional


@dataclass
class Level1:
    oms_id: int
    instrument_id: int
    best_bid: Decimal
    best_offer: Decimal
    last_traded_px: Decimal
    last_traded_qty: Decimal
    last_trade_time: int
    session_open: Decimal
    session_high: Decimal
    session_low: Decimal
    session_close: Decimal
    volume: Decimal
    current_day_volume: Decimal
    current_day_num_trades: int
    current_day_px_change: Decimal
    rolling24hr_notional: Decimal
    rolling24hr_volume: Decimal
    rolling24num_trades: int
    rolling24hr_px_change: Decimal
    time_stamp: str
    current_notional: Optional[Decimal] = None



from dataclasses import dataclass
from typing import Optional


@dataclass
class GetTransactionsRequest:
    from_date: Optional[str] = None
    to_date: Optional[str] = None
    sort: Optional[str] = None
    currency: Optional[str] = None
    page: Optional[int] = None
    page_size: Optional[int] = None



from typing import Any, List
import simplejson as json
from dataclasses import dataclass
from decimal import Decimal


@dataclass
class Level1TickerSummaryMin:
    instrument_id: int
    instrument_symbol: str
    last_traded_px: Decimal
    rolling_24hr_volume: Decimal
    rolling_24hr_px_change: Decimal
    rolling_24hr_px_change_percent: Decimal


def level1_ticker_summary_min_list_from_json_list_str(json_list: str) -> List[Level1TickerSummaryMin]:
    list_of_summaries_lists = json.loads(json_list, use_decimal=True)
    return [level1_ticker_summary_min_from_str(item) for item in list_of_summaries_lists]


def level1_ticker_summary_min_from_str(json_list: List[Any]) -> Level1TickerSummaryMin:
    return Level1TickerSummaryMin(
        instrument_id=json_list[0],
        instrument_symbol=json_list[1],
        last_traded_px=json_list[2],
        rolling_24hr_volume=json_list[3],
        rolling_24hr_px_change=json_list[4],
        rolling_24hr_px_change_percent=json_list[5],
    )



from typing import Optional
from dataclasses import dataclass

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class CancelOrderRequest(WithOMSId):
    account_id: Optional[int] = None
    order_id: Optional[int] = None
    cl_order_id: Optional[int] = None



from dataclasses import dataclass
from decimal import Decimal

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetOrderFeeRequest(WithOMSId):
    account_id: int
    instrument_id: int
    quantity: Decimal
    price: Decimal
    order_type: int
    maker_taker: int
    side: int



from time import sleep
import unittest

from notbank_python_sdk.requests_models.subscribe_level2_request import SubscribeLevel2Request
from notbank_python_sdk.requests_models.unsubscribe_level2_request import UnsubscribeLevel2Request
from notbank_python_sdk.notbank_client import NotbankClient
from tests.test_helper import CallMarker, new_websocket_client_connection


class TestSubscribeLevel2Ticker(unittest.TestCase):
    def setUp(self):
        self._websocket_connection = new_websocket_client_connection()
        self._websocket_connection.connect()
        self._system_client = NotbankClient(self._websocket_connection)

    def test_subscribe_with_instrument_id(self):
        snapshot_marker = CallMarker.create()
        update_marker = CallMarker.create()

        def handle_snapshot():
            snapshot_marker.mark_called()
            print("snapshot n: "+str(snapshot_marker.get_call_count()))

        def handle_update():
            update_marker.mark_called()
            print("update n: " + str(update_marker.get_call_count()))

        self._system_client.subscribe_level_2(
            SubscribeLevel2Request(10, 154),
            lambda tickers: handle_snapshot(),
            lambda tickers: handle_update())
        sleep(80)
        self._system_client.unsubscribe_level_2(
            UnsubscribeLevel2Request(154))
        sleep(3)
        self._websocket_connection.close()
        self.assertTrue(snapshot_marker.was_callled(),
                        'snapshot callback was not called')
        self.assertTrue(update_marker.was_callled(),
                        'update callback was not called')


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class AccountPosition:
    oms_id: int
    account_id: int
    product_symbol: str
    product_id: int
    amount: Decimal
    hold: Decimal
    pending_deposits: Decimal
    pending_withdraws: Decimal
    total_day_deposits: Decimal
    total_month_deposits: Decimal
    total_year_deposits: Decimal
    total_day_deposit_notional: Decimal
    total_month_deposit_notional: Decimal
    total_year_deposit_notional: Decimal
    total_day_withdraws: Decimal
    total_month_withdraws: Decimal
    total_year_withdraws: Decimal
    total_day_withdraw_notional: Decimal
    total_month_withdraw_notional: Decimal
    total_year_withdraw_notional: Decimal
    notional_product_id: int
    notional_product_symbol: str
    notional_value: Decimal
    notional_hold_amount: Decimal
    notional_rate: Decimal
    total_day_transfer_notional: Decimal



import threading
from time import sleep
import logging
import unittest

from notbank_python_sdk.client_connection_factory import new_restarting_websocket_client_connection as create_restarting_websocket_client_connection
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest

from notbank_python_sdk.notbank_client import NotbankClient
from tests.test_helper import load_credentials, TEST_URL


def new_restarting_websocket_client_connection():
    logging.basicConfig()
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    return create_restarting_websocket_client_connection(
        TEST_URL,
        lambda: logger.debug("on open"),
        lambda o1, o2: logger.debug("on close"),
        lambda err: logger.debug("error: " + str(err)),
        lambda msg: logger.debug("message in: " + msg),
        lambda msg: logger.debug("message out: "+msg),
        5)


class TestWebsocketRestarter(unittest.TestCase):
    def setUp(self):
        self._websocket_connection = new_restarting_websocket_client_connection()
        self._websocket_connection.connect()
        self._client = NotbankClient(self._websocket_connection)

    def test_restart_websocket(self):
        credentials = load_credentials()
        self._client.authenticate(AuthenticateRequest(
            credentials.public_key, credentials.secret_key, credentials.user_id))
        while True:
            sleep(10)
            print("threads", [
                  thread.name for thread in threading.enumerate()], "\n")


if __name__ == "__main__":
    unittest.main()



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest
from tests import test_helper


class TestGetAccountInfo(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)


    def test_get_account_info_success(self):
        """Prueba exitosa: oms_id y account_id válidos."""
        request = GetAccountInfoRequest(account_id=7)
        response = self.client.get_account_info(request)
        self.assertEqual(response.account_name, "sample_user")
        self.assertEqual(response.account_type, "Asset")
        self.assertEqual(response.risk_type, "Normal")

    def test_get_account_info_default_account(self):
        """Prueba: account_id no definido, devuelve cuenta por defecto."""
        request = GetAccountInfoRequest()
        response = self.client.get_account_info(request)
        self.assertEqual(response.account_id, 1)
        self.assertEqual(response.account_name, "default_user")


if __name__ == "__main__":
    unittest.main()



from decimal import Decimal
import unittest

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.get_instrument_request import GetInstrumentRequest

from notbank_python_sdk.requests_models.send_order import OrderType, PegPriceType, SendOrderRequest, Side, TimeInForce

from tests import test_helper


class TestSendOrderList(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_valid_order_list(self):
        """Prueba válida: Lista de órdenes correcta."""
        instrument = self.client.get_instrument(GetInstrumentRequest(2))
        request = [
            SendOrderRequest(
                instrument=instrument,
                account_id=185,
                time_in_force=TimeInForce.GTC,
                side=Side.Sell,
                quantity=Decimal("0.02"),
                order_type=OrderType.Limit,
                peg_price_type=PegPriceType.Ask,
                limit_price=Decimal(23436),
                use_display_quantity=False,
            )
        ]

        response = self.client.send_order_list(request)

    def test_empty_order_list(self):
        """Prueba inválida: Lista de órdenes vacía."""
        request = []

        with self.assertRaises(Exception) as context:
            self.client.send_order_list(request)

        # Validaciones
        self.assertEqual(str(context.exception), "Empty order list")


if __name__ == "__main__":
    unittest.main()



from decimal import Decimal
import unittest

from notbank_python_sdk.requests_models.cancel_replace_order_request import CancelReplaceOrderRequest
from notbank_python_sdk.notbank_client import NotbankClient
from tests.test_helper import authenticate_connection, load_credentials, new_rest_client_connection


class TestCancelReplaceOrder(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.connection = new_rest_client_connection()
        cls.credentials = load_credentials('keys.json')
        authenticate_connection(cls.connection, cls.credentials)
        cls.client = NotbankClient(cls.connection)

    def test_cancel_replace_order_success(self):
        request = CancelReplaceOrderRequest(
            order_id_to_replace=123456,
            client_order_id=0,
            order_type="Limit",
            side="Buy",
            account_id=20,
            instrument_id=9,
            limit_price=Decimal(1.363),
            time_in_force=1,
            quantity=Decimal(7322.24),
        )
        response = self.client.cancel_replace_order(request)

        self.assertEqual(response.replacement_order_id, 123457)
        self.assertEqual(response.replacement_cl_ord_id, 0)
        self.assertEqual(response.orig_order_id, 123456)
        self.assertEqual(response.orig_cl_ord_id, 0)

    def test_cancel_replace_order_failed(self):
        request = CancelReplaceOrderRequest(
            order_id_to_replace=123456,
            client_order_id=0,
            order_type="Limit",
            side="Sell",
            account_id=20,
            instrument_id=9,
            limit_price=Decimal(1.363),
            time_in_force=1,
            quantity=Decimal(7322.24),
        )
        try:
            response = self.client.cancel_replace_order(request)
            self.fail("Expected NotbankException but none was raised")
        except:
            pass


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional
from uuid import UUID


@dataclass
class CreateFiatDepositRequest:
    account_id: int
    payment_method: int
    currency: str
    amount: Decimal
    bank_account_id: Optional[UUID] = None
    voucher: Optional[str] = None



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional


@dataclass
class CreateCryptoWithdrawRequest:
    account_id: int
    currency: str
    network: str
    address: str
    amount: Decimal
    memo_or_tag: Optional[str] = None
    otp: Optional[str] = None



from dataclasses import dataclass
from typing import Optional


@dataclass
class AccountInfo:
    oms_id: int
    account_id: int
    account_name: str
    risk_type: str
    account_type: str
    fee_product_type: str
    account_handle: Optional[str] = None
    firm_id: Optional[int] = None
    firm_name: Optional[str] = None
    fee_group_id: Optional[int] = None
    parent_id: Optional[int] = None
    verification_level: Optional[int] = None
    verification_level_name: Optional[str] = None
    credit_tier: Optional[int] = None
    fee_product: Optional[int] = None
    referer_id: Optional[int] = None
    loyalty_product_id: Optional[int] = None
    loyalty_enabled: Optional[bool] = None
    price_tier: Optional[int] = None
    frozen: Optional[bool] = None



from dataclasses import dataclass
from typing import List, Optional
from uuid import UUID
from notbank_python_sdk.models.bank import Bank


@dataclass
class BankAccount:
    id: UUID
    country: str
    bank: Bank
    number: str
    kind: str
    currency: str
    agency: Optional[str]
    dv: Optional[str]
    province: Optional[str]
    pix_type: Optional[str]


@dataclass
class BankAccounts:
    total: int
    data: List[BankAccount]



from dataclasses import dataclass
from typing import List, Optional


@dataclass
class ActivityReport:
    requesting_user: int
    oms_id: int
    report_flavor: str
    create_time: str
    initial_run_time: str
    interval_start_time: str
    interval_end_time: str
    request_status: str
    report_frequency: str
    interval_duration: int
    request_id: str
    last_instance_id: Optional[str]
    account_ids: List[int]



from threading import Lock
from typing import Generic, TypeVar


T = TypeVar("T")


class SynchedValue(Generic[T]):
    _value: T
    _lock: Lock

    def __init__(self, value: T, lock: Lock):
        self._value = value
        self._lock = lock

    @staticmethod
    def create(value: T) -> 'SynchedValue[T]':
        return SynchedValue(value, Lock())

    def get(self) -> T:
        self._lock.acquire()
        value = self._value
        self._lock.release()
        return value

    def set(self, value: T) -> None:
        self._lock.acquire()
        self._value = value
        self._lock.release()



from dataclasses import dataclass
from typing import Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetUserAccountsRequest(WithOMSId):
    user_id: Optional[int] = None



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_account_instrument_statistics_request import GetAccountInstrumentStatisticsRequest
from tests import test_helper


class TestGetAccountInstrumentStatistics(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_account_instrument_statistics_success(self):
        """Prueba exitosa: estadísticas válidas."""
        request = GetAccountInstrumentStatisticsRequest(
            account_id=7,
        )
        response = self.client.get_account_instrument_statistics(request)
        self.assertEqual(len(response), 2)
        self.assertEqual(response[0].instrument_symbol, "BTCUSD")
        self.assertEqual(response[1].instrument_symbol, "ETHUSD")
        self.assertEqual(response[0].monthly_quantity_bought, 0.1602)
        self.assertEqual(response[1].total_month_buys, 6)

    def test_get_account_instrument_statistics_not_found(self):
        """Prueba: account_id inválido, no se encuentra estadísticas."""
        request = GetAccountInstrumentStatisticsRequest(
            account_id=999,
        )
        response = self.client.get_account_instrument_statistics(request)
        self.assertEqual(len(response), 0)


if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.download_document_slice import DownloadDocumentSliceRequest
from tests import test_helper


class TestDownloadDocumentSlice(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_download_document_slice_success(self):
        req = DownloadDocumentSliceRequest(
            "66769552-cacd-471b-bb53-04b1ed1c87f9", 0)
        res = self.client.download_document_slice(req)
        self.assertEqual(res.descriptor_id,
                         "66769552-cacd-471b-bb53-04b1ed1c87f9")
        self.assertEqual(res.status_code, "Success")
        self.assertEqual(res.status_message, "Success")
        self.assertTrue(len(res.base64_bytes) > 0)

    def test_download_document_slice_error(self):
        req = DownloadDocumentSliceRequest("invalid-guid", 1)
        res = self.client.download_document_slice(req)
        self.assertEqual(res.status_code, "Error")
        self.assertEqual(res.status_message, "error")
        self.assertEqual(res.base64_bytes, "")


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetLevel1SummaryMinRequest(WithOMSId):
    instrument_ids: Optional[str] = None



from typing import Any, Dict, List
from notbank_python_sdk.websocket.subscription import Subscription, Unsubscription
from notbank_python_sdk.websocket.websocket_connection import WebsocketConnection


class Resubscriber:
    _id: int
    _active_suscriptions: Dict[int, Subscription[Any]]

    def __init__(self):
        self._id = 0
        self._active_suscriptions = {}

    def save(self, subscription: Subscription[Any]) -> None:
        self._id += 1
        current_id = self._id
        self._active_suscriptions[current_id] = subscription

    def remove(self, unsubscription: Unsubscription[Any]) -> None:
        for callback_id in unsubscription.callback_ids:
            for sub_id in self._active_suscriptions:
                for active_callback in self._active_suscriptions[sub_id].callbacks:
                    if active_callback.id == callback_id:
                        del self._active_suscriptions[sub_id]
                        return

    def _get_subscriptions(self) -> List[Subscription[Any]]:
        return list(self._active_suscriptions.values())

    def resubscribe(self, connection: WebsocketConnection) -> None:
        for subscription in self._get_subscriptions():
            connection.subscribe(subscription)



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class CreateDirectQuoteRequest:
    account_id: int
    from_currency: str
    from_amount: Decimal
    to_currency: str
    operation: str





from dataclasses import dataclass
from decimal import Decimal
from typing import List
from uuid import UUID


@dataclass
class Transaction:
    id: UUID
    legacy_id: str
    currency: str
    direction: int
    amount: Decimal
    fee: str
    balance: Decimal
    address: str
    hash: str
    type: int
    sub_type: int
    status: int
    created_at: str
    updated_at: str


@dataclass
class Transactions:
    total: int
    data: List[Transaction]



from decimal import Decimal
import unittest
from notbank_python_sdk.requests_models.create_crypto_withdraw_request import CreateCryptoWithdrawRequest
from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.get_banks_request import GetBanksRequest


class TestGetBanks(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.credentials = test_helper.load_credentials()        
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_banks(self):
        response = self.client.create_crypto_withdraw(CreateCryptoWithdrawRequest(
          account_id=self.credentials.account_id,
          currency="BTC",
          network="BTC_TEST",
          address="tb1qrekfjtdrffe7r62fr4hspgka8qmzq0t3wu9z87",
          amount=Decimal("0.001"),
          otp="994618"
        ))
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from tests import test_helper


class TestGetEnums(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_enums_success(self):
        response = self.client.get_enums()
        self.assertIsInstance(response, list)
        self.assertGreaterEqual(len(response), 1)
        self.assertEqual(response[0].class_name, "Order")
        self.assertEqual(response[0].property_name, "OrderState")
        self.assertEqual(len(response[0].enums), 2)
        self.assertEqual(response[0].enums[0].name, "Working")
        self.assertEqual(response[0].enums[1].number, 2)


if __name__ == "__main__":
    unittest.main()



from time import sleep
import unittest

from dacite import from_dict, Config
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.models.pong import Pong
from tests import test_helper


class TestPing(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_websocket_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_ping(self):
        self.client.ping()
        self.client.close()


if __name__ == "__main__":
    unittest.main()



import unittest

from notbank_python_sdk.notbank_client import NotbankClient


from notbank_python_sdk.requests_models.get_orders_history import GetOrdersHistoryRequest
from tests import test_helper


class TestGetOrdersHistory(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_orders_history_success(self):
        """Prueba exitosa: Devuelve el historial de órdenes válido."""
        request = GetOrdersHistoryRequest(
            account_id=1,
        )
        response = self.client.get_orders_history(request)

        # Verifica que devuelve correctamente 1 orden
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].order_id, 6713)
        self.assertEqual(response[0].order_state, "FullyExecuted")
        self.assertEqual(response[0].orig_quantity, 0.01)
        self.assertEqual(response[0].avg_price, 6000.0)

    def test_get_orders_history_empty(self):
        """Prueba sin datos: Devuelve una lista vacía."""
        request = GetOrdersHistoryRequest(
            account_id=999,
        )
        response = self.client.get_orders_history(request)

        # Verifica que no se devuelven órdenes
        self.assertEqual(len(response), 0)


# Punto de entrada para ejecutar pruebas
if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_user_report_tickets_by_status import GetUserReportTicketsByStatusRequest
from tests import test_helper


class TestGetUserReportTicketsByStatus(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_user_report_tickets_by_status_multiple(self):
        req = GetUserReportTicketsByStatusRequest(
            request_statuses=["Submitted", "Scheduled"])
        res = self.client.get_user_report_tickets_by_status(req)
        self.assertIsInstance(res, list)
        self.assertEqual(len(res), 2)
        self.assertEqual(res[0].request_status, "Submitted")
        self.assertEqual(res[1].request_status, "Scheduled")
        self.assertEqual(res[1].account_ids, [32])

    def test_get_user_report_tickets_by_status_none(self):
        req = GetUserReportTicketsByStatusRequest()
        res = self.client.get_user_report_tickets_by_status(req)
        self.assertEqual(res, [])


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetLastTradesRequest(WithOMSId):
    instrument_id: int
    count: Optional[int] = 100



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_account_trades import GetAccountTradesRequest
from tests import test_helper


class TestGetAccountTrades(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_account_trades_success(self):
        request = GetAccountTradesRequest(
            account_id=7,
            start_index=0,
            depth=2
        )
        response = self.client.get_account_trades(request)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].trade_id, 916)
        self.assertEqual(response[0].account_name, "sample_user")
        self.assertEqual(response[0].quantity, 0.02)

    def test_get_account_trades_no_results(self):
        request = GetAccountTradesRequest(
            account_id=10
        )
        response = self.client.get_account_trades(request)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 0)


if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.models.level1_ticker import Level1
from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.level1 import GetLevel1Request
from tests import test_helper


class TestGetLevel1(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.client = NotbankClient(connection)

    def test_get_level1_success(self):
        request = GetLevel1Request(instrument_id=1)
        response = self.client.get_level1(request)
        self.assertIsInstance(response, Level1)
        self.assertEqual(response.oms_id, 1)
        self.assertEqual(response.instrument_id, 1)


if __name__ == "__main__":
    unittest.main()



from typing import Dict, List, Optional

from notbank_python_sdk.models.instrument import Instrument
from notbank_python_sdk.models.product import Product


class NotbankClientCache:
    _instrument_by_symbol: Dict[str, Instrument]
    _product_by_symbol: Dict[str, Product]

    def __init__(self):
        self._instrument_by_symbol = {}

    def get_instrument_id(self, symbol: str) -> Optional[int]:
        opt_instrument = self.get_instrument(symbol)
        if opt_instrument is None:
            return opt_instrument
        return opt_instrument.instrument_id

    def get_instrument(self, symbol: str) -> Optional[Instrument]:
        return self._instrument_by_symbol.get(symbol)

    def update_instruments(self, instruments: List[Instrument]) -> None:
        for instrument in instruments:
            self._instrument_by_symbol[instrument.symbol] = instrument

    def get_product(self, symbol: str) -> Optional[Product]:
        return self._product_by_symbol.get(symbol)

    def update_products(self, products: List[Product]) -> None:
        for product in products:
            self._product_by_symbol[product.product] = product



from dataclasses import dataclass
from typing import Union
from uuid import UUID


@dataclass
class DeleteWhiteListedAddressRequest:
    whitelisted_address_id: UUID
    account_id: int
    otp: str


@dataclass
class DeleteWhiteListedAddressRequestInternal:
    account_id: int
    otp: str



from enum import Enum
from decimal import Decimal
from dataclasses import asdict
from typing import Any, Callable, Dict, List, Type, TypeVar, Union
from uuid import UUID

import simplejson as json
from dacite import Config, DaciteError, from_dict as dacite_from_dict

from notbank_python_sdk.core.either import Either
from notbank_python_sdk.error import ErrorCode, NotbankException
from notbank_python_sdk.core.tools import dec_to_str_stripped


T = TypeVar("T")
T1 = TypeVar("T1")
T2 = TypeVar("T2")
CAST_TYPE_FN = Callable[[Any], Any]
TYPE = Type[Any]

SPECIAL_CASES_MAP = {
    "oms_id": "OMSId",
    "api_key": "APIKey",
    "order_id_oco": "OrderIdOCO",
    "requires_2fa": "Requires2FA",
    "two_fa_type": "TwoFAType",
    "two_fa_token": "TwoFAToken",
    "enforce_enable_2fa": "EnforceEnable2FA",
    "use_2fa": "Use2FA",
    "cr": "CR",
    "dr": "DR",
    "parent_id": "ParentID",
    "request_ip": "RequestIP",
    "ipv6a": "IPv6a",
    "ipv6b": "IPv6b",
    "pending_2fa": "Pending2Fa",
    "confirmed_2fa": "Confirmed2Fa",
    "md_update_id": "MDUpdateID",
    "rolling_24_hr_volume": "Rolling24HrVolume",
    "rolling_24_hr_notional": "Rolling24HrNotional",
    "rolling_24_num_trades": "Rolling24NumTrades",
    "rolling_24_hr_px_change": "Rolling24HrPxChange",
    "rolling_24_hr_px_change_percent": "Rolling24HrPxChangePercent",
    "result": "result",
    "errormsg": "errormsg",
    "errorcode": "errorcode",
    "detail": "detail",
    "base64_bytes": "base64Bytes",
    "market_pair": "Market_Pair",
    "m": "m",
    "n": "n",
    "o": "o",
    "i": "i",
}

DEFAULT_CAST: Dict[Type[Any], Any] = {
    Decimal: lambda x: dec_to_str_stripped(x), UUID: lambda x: str(x)}


def _cast_value_or_default(type_hooks: Dict[Type[T1], Callable[[T1], T2]], value: T1) -> Union[T1, T2]:
    cast_fn = type_hooks.get(type(value))
    if cast_fn is None:
        return value
    return cast_fn(value)


def _build_factory(type_hooks: Dict[Type[Any], Any]) -> Callable[[dict], dict]:
    def factory(data: dict) -> dict:
        return dict(
            [entry[0], _cast_value_or_default(type_hooks, entry[1])]
            for entry
            in data
            if entry[1] is not None)
    return factory


def _to_pascal_case(key: str, exclusions: List[str] = [], overrides: Dict[str, str] = {}) -> str:
    converted_key = overrides.get(key)
    if converted_key:
        return converted_key
    pascal_case_key = SPECIAL_CASES_MAP.get(key)
    if pascal_case_key is not None:
        return pascal_case_key
    if key in exclusions:
        return key
    chunks = key.split('_')
    return ''.join(chunk.title() for chunk in chunks)


def to_json_str(data, cast: Dict[Type[Any], Any] = dict()) -> str:
    json_dict = to_dict(data, cast)
    return json.dumps(json_dict, use_decimal=True)


def to_nb_dict(data, cast: Dict[Type[Any], Any] = DEFAULT_CAST) -> dict:
    return to_dict(data, cast, as_snake_case_dict=True)


def to_dict(data, cast: Dict[Type[Any], Any] = DEFAULT_CAST, as_snake_case_dict: bool = False) -> dict:
    if data is None:
        return {}
    dict_factory = _build_factory(cast)
    snake_case_dict = asdict(data, dict_factory=dict_factory)   # type: ignore
    if as_snake_case_dict:
        return snake_case_dict
    return {_to_pascal_case(key): snake_case_dict[key] for key in snake_case_dict}


def from_dict(cls: Type[T1], data, no_pascal_case: List[str] = [], overrides: Dict[str, str] = {}, from_pascal_case: bool = True) -> T1:
    convert_key = get_convert_key_fn(
        no_pascal_case, overrides, from_pascal_case)
    return dacite_from_dict(
        cls,
        data,
        config=Config(
            cast=[Enum],
            convert_key=convert_key,
            type_hooks={
                Decimal: lambda x: Decimal(str(x)),
                float: lambda x: Decimal(str(x)),
                UUID: lambda x: UUID(x)
            }
        )
    )


def get_convert_key_fn(no_pascal_case: List[str], overrides: Dict[str, str], from_pascal_case: bool) -> Callable[[str], str]:
    if from_pascal_case:
        def convert_key(key): return _to_pascal_case(
            key, no_pascal_case, overrides)
        return convert_key
    return lambda x: x


def from_json_str(cls: Type[T1], json_str: str, overrides: Dict[str, str] = {}) -> Either[NotbankException, T1]:
    try:
        data = json.loads(json_str, use_decimal=True)
    except json.JSONDecodeError as e:
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, f"Failed to parse json. {e}"))
    try:
        instance = from_dict(cls, data, overrides=overrides)
    except DaciteError as e:
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, f"Failed to parse json. {e}"))
    return Either.right(instance)


def list_from_json_str(cls: Type[T1], json_str: str) -> Either[NotbankException, List[T1]]:
    try:
        data = json.loads(json_str, use_decimal=True)
    except json.JSONDecodeError as e:
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, f"Failed to parse json. {e}"))
    if not isinstance(data, list):
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, f"Failed to parse json. data is not a list. data is {json_str}"))
    try:
        instance_list = [from_dict(cls, data_elem) for data_elem in data]
    except DaciteError as e:
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, f"Failed to parse json. {e}"))
    return Either.right(instance_list)



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional


@dataclass
class InstrumentVerificationLevelLimits:
    verification_level: int
    oms_id: int
    instrument_id: int
    instrument_name: str
    order_buy_limit: Decimal
    order_sell_limit: Decimal
    daily_buy_limit: Decimal
    daily_sell_limit: Decimal
    monthly_buy_limit: Decimal
    monthly_sell_limit: Decimal
    notional_product_id: int
    order_notional_limit: Decimal
    daily_notional_limit: Decimal
    monthly_notional_limit: Decimal
    yearly_notional_limit: Decimal
    verification_level_name: Optional[str] = None



from dataclasses import dataclass

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetTickerHistoryRequest(WithOMSId):
    instrument_id: int
    interval: int
    from_date: str
    to_date: str



from dataclasses import dataclass
from typing import Union
from uuid import UUID


@dataclass
class ConfirmWhiteListedAddressRequest:
    whitelisted_address_id: UUID
    account_id: int
    sms_code: str


@dataclass
class ConfirmWhiteListedAddressRequestInternal:
    account_id: int
    sms_code: str



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_ticker_history import GetTickerHistoryRequest
from tests import test_helper


class TestGetTickerHistory(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_valid_instrument_id(self):
        """
        Prueba: Obtener historial de ticker con un instrument_id válido.
        """
        request = GetTickerHistoryRequest(
            instrument_id=1,
            interval=60,
            from_date="2023-01-18 01:02:03",
            to_date="2023-08-31 23:59:59",
        )
        response = self.client.get_ticker_history(request)

        # Validaciones
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].instrument_id, 1)
        self.assertEqual(response[0].high, 28432.44)
        self.assertEqual(response[0].volume, 0.0)

    def test_empty_response(self):
        """
        Prueba: Obtener historial de ticker con un instrument_id que no tiene registros.
        """
        request = GetTickerHistoryRequest(
            instrument_id=999,
            interval=60,
            from_date="2023-01-18 01:02:03",
            to_date="2023-08-31 23:59:59",
        )
        response = self.client.get_ticker_history(request)

        # Validaciones
        self.assertEqual(len(response), 0)

    def test_invalid_instrument_id(self):
        """
        Prueba: Obtener historial de ticker con un instrument_id inválido.
        """
        request = GetTickerHistoryRequest(
            instrument_id=0,
            interval=60,
            from_date="2023-01-18 01:02:03",
            to_date="2023-08-31 23:59:59",
        )

        with self.assertRaises(Exception) as context:
            self.client.get_ticker_history(request)

        # Validación del mensaje de error
        self.assertEqual(str(context.exception), "Invalid instrument_id")


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal
from enum import Enum
from typing import Any, Callable, Dict, Generic, Optional, TypeVar

import requests
from notbank_python_sdk.core.ap_data_handler import ApDataHandler

from notbank_python_sdk.core.endpoint_category import EndpointCategory
from notbank_python_sdk.error import ErrorCode, NotbankException, StandardErrorResponse
from dacite.data import Data
from dacite import Config, MissingValueError, from_dict as dacite_from_dict
T = TypeVar('T')


class NBResponseStatus(str, Enum):
    SUCCESS = "success"
    ERROR = "error"


@dataclass
class NBResponse(Generic[T]):
    status: NBResponseStatus
    message: Optional[str] = None
    data: Optional[Any] = None
    total: Optional[int] = None


class ResponseHandler:
    @staticmethod
    def handle_response(endpoint_category: EndpointCategory, parse_response: Callable[[Any], T], response: requests.Response) -> T:
        if endpoint_category == EndpointCategory.AP:
            return ResponseHandler.handle_ap_response(parse_response, response)
        if endpoint_category == EndpointCategory.NB or endpoint_category == EndpointCategory.NB_PAGE:
            return ResponseHandler.handle_nb_response(response, parse_response, endpoint_category)
        raise NotbankException(ErrorCode.CONFIGURATION_ERROR,
                               f"unable to handle server response. handler for endpoint category {endpoint_category} not set")

    @staticmethod
    def handle_nb_response(response: requests.Response, parse_response: Callable[[Any], T], endpoint_category: EndpointCategory) -> T:
        response_data = ResponseHandler.get_response_data(response)
        try:
            nb_response = dacite_from_dict(
                NBResponse,
                response_data,
                config=Config(cast=[Enum]))
            if nb_response.status is NBResponseStatus.ERROR:
                error_message = nb_response.message if nb_response.message else ""
                raise NotbankException(
                    ErrorCode.SERVER_ERROR, f"(http code={response.status_code}) {error_message}")
            data = response_data
            if endpoint_category == EndpointCategory.NB:
                data = nb_response.data
            return parse_response(data)
        except MissingValueError as e:
            raise NotbankException(
                ErrorCode.CONFIGURATION_ERROR,
                f"notbank sdk badly configured. {e}")

    @staticmethod
    def handle_ap_response(parse_response: Callable[[Data], T], response: requests.Response) -> T:
        if response.status_code < 200 or 400 <= response.status_code:
            raise NotbankException(
                ErrorCode.SERVER_ERROR,
                f"http error. (code={response.status_code}) {response.text}")
        response_data = ResponseHandler.get_response_data(response)
        return ApDataHandler.handle_ap_data(parse_response, response_data)

    @staticmethod
    def get_response_data(response: requests.Response):
        try:
            return response.json(parse_float=lambda str_float: Decimal(str_float))
        except requests.JSONDecodeError as e:
            raise NotbankException(
                ErrorCode.INVALID_RESPONSE,
                f"unable to decode json response: {response.text}. {e}")



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class TradeSummary:
    trade_id: int
    price: Decimal
    base_volume: Decimal
    quote_volume: Decimal
    timestamp: str
    type: str



from dataclasses import dataclass

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class SubscribeTickerRequest(WithOMSId):
    instrument_id: int
    interval: int
    include_last_count: int



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.cancel_order import CancelOrder
from tests import test_helper


class TestSendCancelList(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_send_cancel_list_success(self):
        """
        Prueba exitosa: solicitud válida, devuelve 'True'.
        """
        request = [
            CancelOrder(
                order_id=6714,
                account_id=9,
            ),
            CancelOrder(
                order_id=6507,
                account_id=9,
            ),
        ]

        self.client.send_cancel_list(request)

    def test_send_cancel_list_empty_list(self):
        """
        Prueba: lista vacía, devuelve 'False' con mensaje de error.
        """
        request = []
        self.client.send_cancel_list(request)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from enum import Enum
from typing import Optional


class SendOrderStatus(str, Enum):
    ACCEPTED = "Accepted"
    REJECTED = "Rejected"


@dataclass
class SendOrderResponse:
    status: SendOrderStatus
    order_id: Optional[int] = None
    errormsg: Optional[str] = None



from dataclasses import dataclass
from decimal import Decimal
from typing import List, Optional
from enum import IntEnum


class DepositTicketStatus(IntEnum):
    new = 0
    admin_processing = 1
    accepted = 2
    rejected = 3
    system_processing = 4
    fully_processed = 5
    failed = 6
    pending = 7
    confirmed = 8
    aml_processing = 9
    aml_accepted = 10
    aml_rejected = 11
    aml_failed = 12
    limits_accepted = 13
    limits_rejected = 14


@dataclass
class Comment:
    comment_id: int
    entered_by: int
    entered_date_time: str
    entered_date_time_tick: int
    comment: str
    operator_id: int
    oms_id: int
    ticket_id: int


@dataclass
class DepositTicket:
    asset_manager_id: Optional[int] = None
    account_provider_id: Optional[int] = None
    account_id: Optional[int] = None
    asset_id: Optional[int] = None
    account_name: Optional[str] = None
    asset_name: Optional[str] = None
    amount: Optional[Decimal] = None
    notional_value: Optional[Decimal] = None
    notional_product_id: Optional[int] = None
    oms_id: Optional[int] = None
    request_code: Optional[str] = None
    reference_id: Optional[str] = None
    request_ip: Optional[str] = None
    request_user: Optional[int] = None
    request_user_name: Optional[str] = None
    operator_id: Optional[int] = None
    status: Optional[DepositTicketStatus] = None
    fee_amt: Optional[Decimal] = None
    updated_by_user: Optional[int] = None
    updated_by_user_name: Optional[str] = None
    ticket_number: Optional[int] = None
    deposit_info: Optional[str] = None
    reject_reason: Optional[str] = None
    created_timestamp: Optional[str] = None
    last_update_time_stamp: Optional[str] = None
    created_timestamp_tick: Optional[int] = None
    last_update_timestamp_tick: Optional[int] = None
    comments: Optional[List[Comment]] = None
    attachments: Optional[List[str]] = None
    from_address: Optional[str] = None



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.cancel_user_report import CancelUserReportRequest
from tests import test_helper


class TestCancelUserReport(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)
        
    def test_cancel_user_report_success(self):
        req = CancelUserReportRequest(user_report_id="389f244a-b958-4545-a4a7-61a73205b59e")
        self.client.cancel_user_report(req)
        

    def test_cancel_user_report_not_authorized(self):
        req = CancelUserReportRequest(user_report_id="no-perm-guid")
        self.client.cancel_user_report(req)

if __name__ == "__main__":
    unittest.main()
